
Imports System.Globalization

Public NotInheritable Class IeParser
    Private Sub New()
    End Sub

    Public Structure InformationElement
        Public Property ItsNumber() As UShort
            Get
                Return m_ItsNumber
            End Get
            Set(ByVal value As UShort)
                m_ItsNumber = Value
            End Set
        End Property
        Private m_ItsNumber As UShort
        Public Property ItsLength() As UShort
            Get
                Return m_ItsLength
            End Get
            Set(ByVal value As UShort)
                m_ItsLength = Value
            End Set
        End Property
        Private m_ItsLength As UShort
        Public Property ItsData() As Byte()
            Get
                Return m_ItsData
            End Get
            Set(ByVal value As Byte())
                m_ItsData = Value
            End Set
        End Property
        Private m_ItsData As Byte()
    End Structure

#Region "Public Methods"


    ' --------------------------------------------------------------------------------------------
    '  Parse the "Vendor specific info" of the router
    ' --------------------------------------------------------------------------------------------
    Public Structure VendorInfo
        Dim SSID As String
        Dim Brand As String
        Dim Model As String
        Dim Name As String
    End Structure

    Public Shared Function ParseVendorInfo(ByVal ies As Byte()) As VendorInfo
        ParseVendorInfo.SSID = ""
        ParseVendorInfo.Brand = ""
        ParseVendorInfo.Model = ""
        ParseVendorInfo.Name = ""
        Dim informationElements As IEnumerable(Of InformationElement) = BuildInformationElements(ies)
        For Each ie As InformationElement In informationElements
            Dim elem As String = IesToString(ie.ItsData)
            Select Case ie.ItsNumber
                Case 0
                    ParseVendorInfo.SSID = IesToString(ie.ItsData)
                Case 221
                    Dim s As String = IesToString(ie.ItsData)
                    ' -------------------------------------------------------- Brand and Model
                    Dim i1 As Int32 = s.IndexOf(" " + ChrW(33) + "  ")
                    Dim i2 As Int32 = s.IndexOf(" " + ChrW(35) + "  ")
                    Dim i3 As Int32 = s.IndexOf(" " + ChrW(36) + "  ")
                    Dim len1 As Int32 = i2 - i1 - 4
                    Dim len2 As Int32 = i3 - i2 - 4
                    If i1 >= 0 And i2 > i1 And i3 > i2 And len1 > 0 And len2 > 0 Then
                        ParseVendorInfo.Brand = Mid(s, i1 + 5, len1).Trim
                        ParseVendorInfo.Model = Mid(s, i2 + 5, len2).Trim
                    End If
                    ' --------------------------------------------------------- Name
                    If i3 > 0 Then
                        Dim i4 As Int32 = s.IndexOf(" " + ChrW(80) + "  ", i3)
                        If i4 > 0 Then
                            Dim i5 As Int32 = s.IndexOf(" " + ChrW(60) + "  ", i4)
                            Dim len3 As Int32 = i5 - i4 - 4
                            If i4 >= 0 And i5 > i4 And len3 > 0 Then
                                ParseVendorInfo.Name = Mid(s, i4 + 5, len3).Trim
                            End If
                        End If
                    End If
            End Select
        Next
    End Function

    ' --------------------------------------------------------------------------------------------
    '  Parse "N settings"
    ' --------------------------------------------------------------------------------------------
    Public Shared Function Parse(ByVal ies As Byte()) As TypeNSettings

        Dim informationElements As IEnumerable(Of InformationElement) = BuildInformationElements(ies)
        Dim settings As TypeNSettings = New TypeNSettings()
        Dim returnNull As Boolean = True

        For Each ie As InformationElement In informationElements
            Select Case ie.ItsNumber
                Case 45
                    'HT Capabilities
                    ParseHTCapabilities(ie, settings)
                    returnNull = False
                    Exit Select
                Case 61
                    'HT Information
                    ParseHTOperation(ie, settings)
                    returnNull = False
                    Exit Select
            End Select
        Next

        Return If(returnNull, Nothing, settings)
    End Function

    ' --------------------------------------------------------------------------------------------
    '  Parse "AC settings"
    ' --------------------------------------------------------------------------------------------
    Public Shared Function ParseAC(ByVal ies As Byte()) As TypeACSettings

        Dim informationElements As IEnumerable(Of InformationElement) = BuildInformationElements(ies)
        Dim settings As TypeACSettings = New TypeACSettings()
        Dim returnNull As Boolean = True

        For Each ie As InformationElement In informationElements
            Select Case ie.ItsNumber
                Case 45
                    'HT Capabilities
                    ParseHTCapabilities(ie, settings)
                    returnNull = False
                    Exit Select
                Case 61
                    'HT Information
                    ParseHTOperation(ie, settings)
                    returnNull = False
                    Exit Select
                Case 191
                    'VHT Capabilities
                    ParseVHTCapabilities(ie, settings)
                    Exit Select
                Case 192
                    'VHT Operation
                    ParseVHTOperation(ie, settings)
                    Exit Select
            End Select
        Next

        Return If(returnNull, Nothing, settings)
    End Function


    Private Shared Sub ParseVHTOperation(ByVal ie As InformationElement, ByVal settings As TypeACSettings)
        settings.Operations = New TypeACSettings.VHTOperations()

        Dim operations As Byte() = New Byte(3) {}
        Array.Copy(ie.ItsData, 0, operations, 0, 3)

        Dim basicMCSSet As Byte() = New Byte(7) {}
        Array.Copy(ie.ItsData, 3, basicMCSSet, 0, 2)

        settings.Operations.ChannelWidth = DirectCast([Enum].Parse(GetType(TypeACSettings.VHTOperations.VHTChannelWidth), operations(0).ToString(CultureInfo.InvariantCulture)), TypeACSettings.VHTOperations.VHTChannelWidth)
    End Sub

    Private Shared Sub ParseVHTCapabilities(ByVal ie As InformationElement, ByVal settings As TypeACSettings)
        settings.Capabilities = New TypeACSettings.VHTCapabilities()

        Dim capabilities As Byte() = New Byte(3) {}
        Array.Copy(ie.ItsData, 0, capabilities, 0, 4)

        Dim supportedMCS As Byte() = New Byte(7) {}
        Array.Copy(ie.ItsData, 4, supportedMCS, 0, 8)

        Dim supportedChannelWidth As Int32 = (capabilities(0) And &HC) >> 2

        Select Case supportedChannelWidth
            Case 1
                settings.Capabilities.Supports160Mhz = True
                Exit Select
            Case 2
                settings.Capabilities.Supports80Plus80Mhz = True
                settings.Capabilities.Supports160Mhz = True
                Exit Select
        End Select

        settings.Capabilities.ShortGi160MHz = (capabilities(0) And &H40) = &H40
        settings.Capabilities.ShortGi80MHz = (capabilities(0) And &H20) = &H20

        settings.Capabilities.MaxReceiveRate = BitConverter.ToUInt16(supportedMCS, 2)
        settings.Capabilities.MaxTransmitRate = BitConverter.ToUInt16(supportedMCS, 6)
    End Sub

    Private Shared Sub ParseHTOperation(ByVal ie As InformationElement, ByVal settings As TypeNSettings)
        Const channel As Integer = 0
        Const subset1 As Integer = 1

        'Primary channel
        settings.PrimaryChannel = ie.ItsData(0)

        'Secondary channel location
        settings.SecondaryChannelLower = (ie.ItsData(channel) And &H3) = &H3

        'Check if there is no secondary channel and set 40MHz to false
        If settings.Is40Mhz Then
            settings.Is40Mhz = (ie.ItsData(subset1) And &H3) = &H3 OrElse (ie.ItsData(subset1) And &H1) = &H1
        End If
    End Sub

    Private Shared Sub ParseHTCapabilities(ByVal ie As InformationElement, ByVal settings As TypeNSettings)
        settings.Is40Mhz = ((ie.ItsData(0) And &H2) = &H2)

        settings.ShortGi20MHz = (ie.ItsData(0) And &H20) = &H20
        settings.ShortGi40MHz = (ie.ItsData(0) And &H40) = &H40

        'Get supported MCS indexes 
        '1 bit per index

        Dim bits As Byte() = New Byte(3) {}
        'Array.ConstrainedCopy(ies, index + 5, bits, 0, 4);
        Array.Copy(ie.ItsData, 4, bits, 0, 4)

        Dim b As New BitArray(bits)
        'settings.Rates = new List<double>();

        'The MCS indexes are in little endian,
        'so this loop will start at the lowest rates
        For i As Integer = 0 To b.Length - 1
            'If the MCS index bit is 0, skip it
            If b(i) = False Then
                Continue For
            End If

            'Add the rate
            settings.Rates.Add(McsSet.GetSpeed(CUInt(i), settings.ShortGi20MHz, settings.ShortGi40MHz, settings.Is40Mhz))
        Next
    End Sub

    Private Shared Function BuildInformationElements(ByVal ies As Byte()) As IEnumerable(Of InformationElement)

        On Error Resume Next

        Dim informationElements As List(Of InformationElement) = New List(Of InformationElement)

        Dim index As Int32 = 0

        While index < ies.Length
            Dim ie As InformationElement = New InformationElement
            ie.ItsNumber = ies(index)
            ie.ItsLength = ies(index + 1)
            ie.ItsData = New Byte(ie.ItsLength - 1) {}
            Array.Copy(ies, index + 2, ie.ItsData, 0, ie.ItsLength)

            informationElements.Add(ie)

            index += ie.ItsLength + 2
        End While
        Return informationElements
    End Function


    Public Class TypeNSettings
        Public Is40Mhz As Boolean
        Public ShortGi20MHz As Boolean
        Public ShortGi40MHz As Boolean
        Public PrimaryChannel As UInteger
        Public SecondaryChannelLower As Boolean

        'public uint MaxMcs;
        Public Rates As List(Of Double)

        Public Sub New()
            Rates = New List(Of Double)()
        End Sub

        Public Sub New(ByVal settings As TypeNSettings)
            Is40Mhz = settings.Is40Mhz
            ShortGi20MHz = settings.ShortGi20MHz
            ShortGi40MHz = settings.ShortGi40MHz
            PrimaryChannel = settings.PrimaryChannel
            SecondaryChannelLower = settings.SecondaryChannelLower
            'MaxMcs = settings.MaxMcs;
            Rates = settings.Rates
        End Sub

        Public Overrides Function Equals(ByVal obj As Object) As Boolean
            If TypeOf obj Is TypeNSettings Then
                Dim [set] As TypeNSettings = DirectCast(obj, TypeNSettings)
                Dim yes As Boolean = [set].Is40Mhz = Is40Mhz
                yes = yes And [set].ShortGi20MHz = ShortGi20MHz
                yes = yes And [set].ShortGi40MHz = ShortGi40MHz
                yes = yes And [set].PrimaryChannel = PrimaryChannel
                yes = yes And [set].SecondaryChannelLower = SecondaryChannelLower
                'Don't compare rates

                Return yes
            End If
            Return False
        End Function
    End Class


    Public Class TypeACSettings
        Inherits TypeNSettings
        Public Class VHTCapabilities
            Public ShortGi80MHz As Boolean
            Public ShortGi160MHz As Boolean
            Public Supports160Mhz As Boolean
            Public Supports80Plus80Mhz As Boolean
            Public MaxReceiveRate As UShort
            Public MaxTransmitRate As UShort
        End Class

        Public Class VHTOperations
            Public Enum VHTChannelWidth As Byte
                TwentyOrForty = &H0
                Eighty = &H1
                OneSixty = &H2
                EightyPlusEighty = &H3
            End Enum

            Public ChannelWidth As VHTChannelWidth
        End Class

        Public Property Capabilities() As VHTCapabilities
            Get
                Return m_Capabilities
            End Get
            Set(ByVal value As VHTCapabilities)
                m_Capabilities = value
            End Set
        End Property
        Private m_Capabilities As VHTCapabilities
        Public Property Operations() As VHTOperations
            Get
                Return m_Operations
            End Get
            Set(ByVal value As VHTOperations)
                m_Operations = value
            End Set
        End Property
        Private m_Operations As VHTOperations
    End Class


#End Region

#Region "Private Methods"

    Private Class McsSet

        '20MHz long GI
        '6.5
        '19.5
        Private Shared ReadOnly LGiTable20 As Single() = {6.0F, _
                                                          13.0F, _
                                                          19.0F, _
                                                          26.0F, _
                                                          39.0F, _
                                                          52.0F, _
                                                          58.0F, _
                                                          65.0F}

        '20MHz short GI
        '7.2
        '14.4
        '21.7
        '28.9
        '43.3
        '57.8
        '72.2
        Private Shared ReadOnly SGiTable20 As Single() = {7.0F, _
                                                          14.0F, _
                                                          22.0F, _
                                                          29.0F, _
                                                          43.0F, _
                                                          58.0F, _
                                                          65.0F, _
                                                          72.0F}

        '40MHz long GI
        '13.5
        '40.5
        Private Shared ReadOnly LGiTable40 As Single() = {13.0F, _
                                                          27.0F, _
                                                          40.0F, _
                                                          54.0F, _
                                                          81.0F, _
                                                          108.0F, _
                                                          121.0F, _
                                                          135.0F}

        '40MHz short GI
        Private Shared ReadOnly SGiTable40 As Single() = {15.0F, _
                                                          30.0F, _
                                                          45.0F, _
                                                          60.0F, _
                                                          90.0F, _
                                                          120.0F, _
                                                          135.0F, _
                                                          150.0F}

        Public Shared Function GetSpeed(ByVal index As UInteger, ByVal shortGi20MHz As Boolean, ByVal shortGi40MHz As Boolean, ByVal fortyMHz As Boolean) As Single
            Dim output As Single

            If index > 32 Then
                Return 0.0F
            End If
            Dim streams As Integer = 0

            If index >= 0 AndAlso index < 8 Then
                streams = 1
            ElseIf index >= 8 AndAlso index < 16 Then
                streams = 2
                index -= 8UI
            ElseIf index >= 16 AndAlso index < 24 Then
                streams = 3
                index -= 16UI
            ElseIf index >= 24 AndAlso index < 32 Then
                streams = 4
                index -= 24UI
            End If

            If fortyMHz Then
                If shortGi40MHz Then
                    output = SGiTable40(CInt(index))
                    output *= streams
                Else
                    output = LGiTable40(CInt(index))
                    output *= streams
                End If
            Else
                '20 MHz channel
                If shortGi20MHz Then
                    output = SGiTable20(CInt(index))
                    output *= streams
                Else
                    output = LGiTable20(CInt(index))
                    output *= streams
                End If
            End If

            Return output
        End Function
    End Class

#End Region

End Class


