﻿
Imports System.Drawing.Drawing2D


Friend Enum GraphTypes As Int32
    Quality
    Signal
    Detections
    QualityAndDetections
End Enum

Public Class SignalGraph

    Friend NumGraphs As Int32
    Friend SelectedGraph As Int32 = -1

    Friend GraphColors(7) As Color

    Friend GraphType As GraphTypes

    Friend xScale As Int32 = 1

    Friend hpix As Int32
    Friend wpix As Int32

    Private BackColor As Color = Color.FromArgb(255, 245, 230)
    Private scalePen1 As Pen = New Pen(Color.FromArgb(140, 120, 120))
    Private scalePen2 As Pen = New Pen(Color.FromArgb(200, 190, 190))
    Private scalePen3 As Pen = New Pen(Color.FromArgb(220, 210, 210))
    Private scalePen4 As Pen = New Pen(Color.FromArgb(230, 230, 230))

    'Private tracePen1 As Pen = New Pen(Color.FromArgb(50, 0, 0), 2)
    'Private tracePen2 As Pen = New Pen(Color.FromArgb(0, 90, 0), 2)

    Dim pen1 As Pen = New Pen(Color.Black)

    Private textBrush As Brush = New SolidBrush(Color.FromArgb(40, 40, 40))
    'Private hilightBack As Brush = New SolidBrush(Color.FromArgb(90, 255, 255, 100))
    'Private hilightText As Brush = New SolidBrush(Color.FromArgb(50, 50, 50))
    Private textFont As Font = New Font("Microsoft Sans Serif", 8, FontStyle.Regular)
    'New Font("Arial", 10, FontStyle.Bold)

    Friend NumGraphColors As Int32 = 8

    Friend Sub InitGraphColors()
        For i As Int32 = 0 To NumGraphColors - 1
            Dim c As Color
            Select Case i
                Case 0 : c = Color.FromArgb(50, 90, 255)  ' blue
                Case 1 : c = Color.FromArgb(230, 20, 40)  ' red
                Case 2 : c = Color.FromArgb(0, 170, 40)   ' green
                Case 3 : c = Color.FromArgb(130, 0, 200)  ' pink
                Case 4 : c = Color.FromArgb(240, 80, 0)   ' orange
                Case 5 : c = Color.FromArgb(30, 30, 30)   ' dark gray
                Case 6 : c = Color.FromArgb(0, 110, 0)    ' dark green
                Case 7 : c = Color.FromArgb(150, 0, 0)    ' dark red
            End Select
            GraphColors(i) = c
        Next
    End Sub

    Friend Sub InitPictureboxImage(ByVal pbox As PictureBox)
        With pbox
            If .ClientSize.Width < 1 Or .ClientSize.Height < 1 Then Return
            If .Image Is Nothing OrElse .Image.Width <> .ClientSize.Width _
                                 OrElse .Image.Height <> .ClientSize.Height Then
                .Image = New Bitmap(.ClientRectangle.Width, _
                                    .ClientRectangle.Height, _
                                    Imaging.PixelFormat.Format24bppRgb)
            End If
        End With
    End Sub

    Friend Sub Update(ByVal pbox As PictureBox)
        If Form1.ViewMode = "Channels" Then
            UpdateChannels(pbox)
        Else
            UpdateGraphs(pbox)
        End If
    End Sub

    Private Function CheckedSSID(ByVal SSID As String) As Boolean
        For i As Int32 = 0 To NetworkDataArray.Length - 1
            If NetworkDataArray(i).SSID = SSID Then
                Return NetworkDataArray(i).Checked
            End If
        Next
        Return False
    End Function


    ' ==============================================================================
    '  Update GRAPHS
    ' ==============================================================================

    Friend Sub UpdateGraphs(ByVal pbox As PictureBox)
        '
        'Dim sw1 As Diagnostics.Stopwatch = New Diagnostics.Stopwatch : sw1.Start()
        ' --------------------------------------------------------------------- 
        If Not pbox.Visible Then Return
        InitPictureboxImage(pbox)
        ' --------------------------------------------------------------------- 
        Dim v1 As Single
        Dim i As Int32
        Dim x As Single
        Dim y, y2 As Single
        Dim oldx(99) As Int32
        Dim oldy(99) As Single
        hpix = pbox.Image.Height
        wpix = pbox.Image.Width - 41
        Dim ArrayReadIndex As Int32
        If hpix < 1 Or wpix < 1 Then Return
        ' ---------------------------------------------------------------------
        Dim g As Graphics = Graphics.FromImage(pbox.Image)
        g.Clear(BackColor)
        g.SmoothingMode = Drawing2D.SmoothingMode.HighQuality
        g.TextRenderingHint = Drawing.Text.TextRenderingHint.ClearTypeGridFit
        ' --------------------------------------------------------------------- array index
        ArrayReadIndex = CInt(DataArray1.GetWriteIndex - wpix * xScale)
        ' --------------------------------------------------------------------- stabilize and suppress aliasing effect
        ArrayReadIndex = CInt(xScale * CInt(ArrayReadIndex / xScale))
        ' --------------------------------------------------------------------- draw scale x
        y = PercentToY(0)
        y2 = PercentToY(100)
        For x = wpix To 0 Step -1
            i = CInt(x * xScale + ArrayReadIndex)
            If i Mod 600 = 0 Then
                g.DrawLine(scalePen1, x, y, x, y2)
            ElseIf i Mod 100 = 0 Then
                g.DrawLine(scalePen3, x, y, x, y2)
            ElseIf i Mod 10 = 0 Then
                If xScale < 5 Then
                    g.DrawLine(scalePen4, x, y, x, y2)
                End If
            End If
        Next
        g.DrawLine(scalePen2, wpix, y, wpix, y2)
        ' --------------------------------------------------------------------- draw scale y
        For i = 0 To 100 Step 5
            y = PercentToY(i)
            If i Mod 10 = 0 Then
                g.DrawLine(scalePen2, 0, y, wpix + 2, y)
            ElseIf i Mod 5 = 0 Then
                g.DrawLine(scalePen4, 0, y, wpix, y)
            End If
        Next
        For i = 10 To 90 Step 10
            y = PercentToY(i)
            If GraphType = GraphTypes.Signal Then
                g.DrawString((i - 100).ToString + "dB", textFont, textBrush, wpix + 7, y - 7)
            Else
                g.DrawString(i.ToString + " %", textFont, textBrush, wpix + 8, y - 7)
            End If
        Next

        ' --------------------------------------------------------------------- draw all waves
        For n As Int32 = 0 To 99
            oldx(n) = -2
            oldy(n) = -1
        Next

        For ix As Int32 = 0 To wpix - 2
            ArrayReadIndex += xScale
            If ArrayReadIndex < 0 Then Continue For

            With DataArray1.GetElement(CInt(ArrayReadIndex))
                If .Networks IsNot Nothing Then

                    For net As Int32 = 0 To .Networks.Length - 1

                        If CheckedSSID(.Networks(net).ssid) Then

                            Dim detections As Single = .Networks(net).DetectionPercentual

                            ' ------------------------------------------------------------- select data
                            v1 = SelectData(.Networks(net))

                            ' ------------------------------------------------------------- calc Yposition
                            y = PercentToY(v1)

                            If oldx(net) >= 0 And oldy(net) >= 0 Then
                                pen1.Color = GraphColors(net Mod NumGraphColors)
                                If SelectedGraph < 0 Then
                                    pen1.Width = 2
                                Else
                                    pen1.Width = If(SelectedGraph = net, 4, 1)
                                End If
                                g.DrawLine(pen1, oldx(net), oldy(net), ix, y)
                            End If

                            ' ------------------------------------------------------------- wait the second value
                            If oldx(net) < 0 Then
                                oldx(net) += 1
                            Else
                                oldx(net) = ix
                                oldy(net) = y
                            End If

                        End If
                    Next

                End If
            End With
        Next
        ' --------------------------------------------------------------------- show
        pbox.Invalidate()
    End Sub


    ' ==============================================================================
    '  Update CHANNELS
    ' ==============================================================================

    Friend Sub UpdateChannels(ByVal pbox As PictureBox)
        '
        'Dim sw1 As Diagnostics.Stopwatch = New Diagnostics.Stopwatch : sw1.Start()
        ' --------------------------------------------------------------------- 
        If Not pbox.Visible Then Return
        InitPictureboxImage(pbox)
        ' --------------------------------------------------------------------- 
        Dim v1 As Single
        Dim i As Int32
        Dim x As Single
        Dim y, y2, y3 As Single
        hpix = pbox.Image.Height
        wpix = pbox.Image.Width - 41
        If hpix < 1 Or wpix < 1 Then Return
        ' --------------------------------------------------------------------- graphics
        Dim g As Graphics = Graphics.FromImage(pbox.Image)
        g.Clear(BackColor)
        g.SmoothingMode = Drawing2D.SmoothingMode.HighQuality
        g.TextRenderingHint = Drawing.Text.TextRenderingHint.ClearTypeGridFit
        ' --------------------------------------------------------------------- draw scale x
        y = PercentToY(100)
        y2 = PercentToY(8)
        y3 = PercentToY(5)
        For i = -2 To 17
            x = ChannelToX(i)
            g.DrawLine(scalePen2, x, y, x, y2)
            If i >= 1 And i <= 13 Then
                g.DrawString(i.ToString, textFont, textBrush, x - If(i > 9, 9, 5), y3)
            ElseIf i = 15 Then
                g.DrawString("14", textFont, textBrush, x - If(i > 9, 9, 5), y3)
            End If
        Next
        ' --------------------------------------------------------------------- draw scale y
        For i = 10 To 100
            y = PercentToY(i)
            If i Mod 10 = 0 Then
                g.DrawLine(scalePen2, 4, y, wpix + 5, y)
            ElseIf i Mod 5 = 0 Then
                g.DrawLine(scalePen4, 4, y, wpix, y)
            End If
        Next
        For i = 10 To 90 Step 10
            y = PercentToY(i)
            If GraphType = GraphTypes.Signal Then
                g.DrawString((i - 100).ToString + "dB", textFont, textBrush, wpix + 7, y - 7)
            Else
                g.DrawString(i.ToString + " %", textFont, textBrush, wpix + 8, y - 7)
            End If
        Next
        ' --------------------------------------------------------------------- draw channels
        If NetworkDataArray IsNot Nothing Then

            For net As Int32 = 0 To NetworkDataArray.Length - 1

                If NetworkDataArray(net).Checked Then

                    ' ------------------------------------------------------------- select data
                    v1 = SelectData(NetworkDataArray(net))
                    If v1 < 10 Then v1 = 10
                    Dim detections As Single = NetworkDataArray(net).DetectionPercentual
                    ' ------------------------------------------------------------- calc color
                    pen1.Color = GraphColors(net Mod NumGraphColors)
                    ' ------------------------------------------------------------- calc width
                    If SelectedGraph < 0 Then
                        pen1.Width = 2
                    Else
                        pen1.Width = If(SelectedGraph = net, 4, 1)
                    End If
                    ' ------------------------------------------------------------- calc Yposition
                    y = PercentToY(v1)
                    ' ------------------------------------------------------------- calc Xposition
                    CalcChannelPositions(net)
                    ' ------------------------------------------------------------- draw channel
                    g.DrawLine(pen1, ChPos.x1, y2, ChPos.x2, y)
                    g.DrawLine(pen1, ChPos.x2, y, ChPos.x3, y)
                    g.DrawLine(pen1, ChPos.x3, y, ChPos.x4, y2)

                    ' ------------------------------------------------------------- draw text
                    Dim s As SizeF = g.MeasureString(NetworkDataArray(net).SSID, textFont, 999)
                    pen1.Color = GraphColors(net Mod NumGraphColors)
                    g.DrawString(NetworkDataArray(net).SSID, textFont, New SolidBrush(pen1.Color), (ChPos.x2 + ChPos.x3 - s.Width) / 2.0F, y + 3)
                End If
            Next
        End If
        ' --------------------------------------------------------------------- show
        pbox.Invalidate()
    End Sub


    Friend Sub SelectNet(ByVal x As Int32, ByVal y As Int32)
        '
        Dim MinDistance As Single = 9999
        Dim NearestNet As Int32 = -1
        '
        If Form1.ViewMode = "Channels" Then
            If NetworkDataArray IsNot Nothing Then
                For net As Int32 = 0 To NetworkDataArray.Length - 1
                    If NetworkDataArray(net).Checked Then
                        ' ------------------------------------------------------------- select data
                        Dim v1 As Single = SelectData(NetworkDataArray(net))
                        'Debug.Print(ypos.ToString + "   " + v1.ToString + vbCrLf)
                        ' ------------------------------------------------------------- 
                        CalcChannelPositions(net)
                        If x > ChPos.x2 And x < ChPos.x3 Then
                            Dim distance As Single = Math.Abs(y - PercentToY(v1))
                            If distance < MinDistance Then
                                MinDistance = distance
                                NearestNet = net
                            End If
                        End If
                    End If
                Next
                'Debug.Print(vbCrLf)
            End If
        Else
            Dim Xpixel As Int32 = x - wpix
            Dim sample As Int32 = Xpixel * xScale
            Dim ArrayIndex As Int32 = DataArray1.GetWriteIndex + sample
            ' ------------------------------------------------------------------------------------
            For ai As Int32 = ArrayIndex - 10 * xScale To ArrayIndex + 10 * xScale
                With DataArray1.GetElement(ai)
                    If .Networks IsNot Nothing Then
                        For net As Int32 = 0 To .Networks.Length - 1
                            If NetworkDataArray(net).Checked Then
                                ' ------------------------------------------------------------- select data
                                Dim v1 As Single = SelectData(.Networks(net))
                                'Debug.Print(y.ToString + "   " + PercentToY(v1).ToString)
                                ' ------------------------------------------------------------- 
                                Dim distance As Single = Math.Abs(y - PercentToY(v1))
                                If distance < MinDistance Then
                                    MinDistance = distance
                                    NearestNet = net
                                End If
                            End If
                        Next
                        'Debug.Print(vbCrLf)
                    End If
                End With
            Next
        End If
        '
        If MinDistance < 25 Then
            Form1.SelectListLineFromMAC(NetworkDataArray(NearestNet).MAC_Address)
        Else
            Form1.ListView1.SelectedItems.Clear()
        End If
        '
        Update(Form1.pbox_Graph)
    End Sub

    Private Structure ChannelPositions_Structure
        Dim x1 As Single
        Dim x2 As Single
        Dim x3 As Single
        Dim x4 As Single
    End Structure
    Private ChPos As ChannelPositions_Structure

    Private Sub CalcChannelPositions(ByVal net As Int32)
        Dim ch As String = NetworkDataArray(net).Channel
        If ch Is Nothing Then Return
        Dim chmin As Int32 = CInt(Val(ch))
        Dim chmax As Int32 = chmin
        ' ------------------------------------------- eventually extract the second channel
        If ch.Contains("+") Then
            chmax = CInt(Val(Mid(ch, 4)))
            If chmin > chmax Then Swap(chmin, chmax)
        End If
        ' ------------------------------------------- limit max channel to 14
        If chmin > 14 Then chmin = 14
        If chmax > 14 Then chmax = 14
        ' ------------------------------------------- channel 14 frequency is shifted right to 15
        If chmin = 14 Then chmin = 15
        If chmax = 14 Then chmax = 15
        ' ------------------------------------------- calc X positions
        ChPos.x1 = ChannelToX(chmin - 2)
        ChPos.x2 = ChannelToX(chmin - 1.5F)
        ChPos.x3 = ChannelToX(chmax + 1.5F)
        ChPos.x4 = ChannelToX(chmax + 2)
    End Sub

    Private Function PercentToY(ByVal percent As Single) As Single
        Return hpix - 3 - (hpix - 8) * percent / 100
    End Function

    Private Function ChannelToX(ByVal channel As Single) As Single
        Return 4 + (channel + 1) * wpix / 18.0F
    End Function

    Friend Function SelectData(ByVal nd As NetworkData) As Single
        Dim d As Single = nd.DetectionPercentual
        Select Case GraphType
            Case GraphTypes.Quality : Return nd.Quality
            Case GraphTypes.Signal : Return nd.RSSI + 100
            Case GraphTypes.Detections : Return d
            Case GraphTypes.QualityAndDetections : Return (nd.Quality + d) / 2
        End Select
    End Function

    Private Function SelectData(ByVal nd As DataArray.NetworkGraphData) As Single
        Dim d As Single = nd.DetectionPercentual
        Select Case GraphType
            Case GraphTypes.Quality : Return nd.Quality
            Case GraphTypes.Signal : Return nd.RSSI + 100
            Case GraphTypes.Detections : Return d
            Case GraphTypes.QualityAndDetections : Return (nd.Quality + d) / 2
        End Select
    End Function

End Class
